﻿using System;
using com.epson.bank.driver;

namespace TM_S9000Sample
{
    class CApp
    {
        public const byte EXEC_SCAN_CHECK   = 0;
        public const byte EXEC_SCAN_CARD    = 1;
        public const byte EXEC_PRINT        = 2;

        public delegate void CallbackProcProcessError(ErrorCode errResult);
        public delegate void CallbackProcImage();
        public delegate void CallbackProcMicr();
        public delegate void CallbackProcPhysicalEndorse(uint unTransactionNumber);
        public delegate void CallbackProcElectronicEndorse();

        private CallbackProcProcessError        m_cbFuncProcessErrorCB       = null;
        private CallbackProcImage               m_cbFuncImageCB              = null;
        private CallbackProcMicr                m_cbFuncMicrCB               = null;
        private CallbackProcPhysicalEndorse     m_cbFuncPhysicalEndorseCB    = null;
        private CallbackProcElectronicEndorse   m_cbFuncElectronicEndorseCB  = null;

        private CProperty_1 m_objProperty   = null;
        private CScan       m_objScan       = null;
        private CMicr       m_objMicr       = null;
        private CImage_1    m_objImage      = null;
        private CEndorse    m_objEndorse    = null;
        private COption     m_objOption     = null;

        private byte        m_byExecType;
        private bool        m_bOpenDevice;

        public CApp()
        {
            m_objProperty   = new CProperty_1();
            m_objScan       = new CScan();
            m_objMicr       = new CMicr();
            m_objImage      = new CImage_1();
            m_objEndorse    = new CEndorse();
            m_objOption     = new COption();

            m_byExecType                = EXEC_SCAN_CHECK;
        }

        // Initialize
        public ErrorCode InitDevice()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            m_bOpenDevice = false;

            try
            {
                // Connect to the device
                m_objProperty.SetMfDevice(new MFDevice());
                errResult = m_objProperty.GetMfDevice().OpenMonPrinter(OpenType.TYPE_PRINTER, "TM-S9000U,TM-S2000U");
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }
                m_bOpenDevice = true;

                // Register callback functions for notification of the reading status
                m_objProperty.GetMfDevice().SCNMICRStatusCallback += new MFDevice.SCNMICRStatusCallbackHandler(ScanStatus);
                errResult = m_objProperty.GetMfDevice().SCNMICRSetStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }

                // Register callback functions for notification of the endorsement data acceptance
                m_objProperty.GetMfDevice().StartEndorsementStatusCallback += new MFDevice.StartEndorsementStatusCallbackHandler(StartEndorsement);
                errResult = m_objProperty.GetMfDevice().StartEndorsementSetStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }

                // Register callback functions for notification of the endorsement data close
                m_objProperty.GetMfDevice().EndEndorsementStatusCallback += new MFDevice.EndEndorsementStatusCallbackHandler(EndEndorsement);
                errResult = m_objProperty.GetMfDevice().EndEndorsementSetStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }

                m_objScan.Init(m_objProperty);
                m_objMicr.Init(m_objProperty);
                m_objImage.Init(m_objProperty);
                m_objOption.Init(m_objProperty);
                //Step4
                m_objEndorse.Init(m_objProperty);

            }
            catch
            {
                return ErrorCode.ERR_UNKNOWN;
            }

            return errResult;
        }

        // Post process
        public ErrorCode ExitDevice()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            if (m_bOpenDevice == true)
            {
                // Cancel the reading status callback
                errResult = m_objProperty.GetMfDevice().SCNMICRCancelStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }
                m_objProperty.GetMfDevice().SCNMICRStatusCallback -= new MFDevice.SCNMICRStatusCallbackHandler(ScanStatus);

                // Step4
                // Cancel the endorsement data acceptance callback
                errResult = m_objProperty.GetMfDevice().StartEndorsementCancelStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }
                m_objProperty.GetMfDevice().StartEndorsementStatusCallback -= new MFDevice.StartEndorsementStatusCallbackHandler(StartEndorsement);

                // Cancel the endorsement data close callback
                errResult = m_objProperty.GetMfDevice().EndEndorsementCancelStatusBack();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }
                m_objProperty.GetMfDevice().EndEndorsementStatusCallback -= new MFDevice.EndEndorsementStatusCallbackHandler(EndEndorsement);


                // Disconnect from device
                errResult = m_objProperty.GetMfDevice().CloseMonPrinter();
                if (errResult != ErrorCode.SUCCESS)
                {
                    return errResult;
                }
                m_bOpenDevice = false;
            }

            return errResult;
        }

        // Register callback function for notification of the MICR data
        public void SetMicrCallback(CallbackProcMicr cbMicrCB)
        {
            m_cbFuncMicrCB = cbMicrCB;
        }

        // Register callback function for notification of the image data
        public void SetImageCallback(CallbackProcImage cbImageCB)
        {
            m_cbFuncImageCB = cbImageCB;
        }

        // Register callback function for notification of error during image acquisition
        public void SetProcessErrorCallback(CallbackProcProcessError cbProcessErrorCB)
        {
            m_cbFuncProcessErrorCB = cbProcessErrorCB;
        }

        // Set MICR settings
        public void SetMicrParam(ref CMicrParam objMicrParam)
        {
            if (objMicrParam != null)
            {
                m_objProperty.SetFont(objMicrParam.GetFont());
                m_objProperty.SetParsing(objMicrParam.GetParsing());
                m_objProperty.SetClearSpace(objMicrParam.GetClearSpace());
            }
        }

        // Get MICR settings
        public void GetMicrParam(ref CMicrParam objMicrParam)
        {
            if (objMicrParam != null)
            {
                objMicrParam.SetFont(m_objProperty.GetFont());
                objMicrParam.SetParsing(m_objProperty.GetParsing());
                objMicrParam.SetClearSpace(m_objProperty.GetClearSpace());
            }
        }

        // Set scanning parameters
        public void SetScanParam(ref CScanParam objScanParam)
        {
            if (objScanParam != null)
            {
                m_objProperty.SetScanMedia(objScanParam.GetScanMedia());
                m_objProperty.SetImageChannel(objScanParam.GetImageChannel());
                m_objProperty.SetRGBColorDepth(objScanParam.GetRGBColorDepth());
                m_objProperty.SetIRColorDepth(objScanParam.GetIRColorDepth());
                m_objProperty.SetResolution(objScanParam.GetResolution());
            }
        }

        // Get scanning parameters
        public void GetScanParam(ref CScanParam objScanParam)
        {
            if (objScanParam != null)
            {
                objScanParam.SetScanMedia(m_objProperty.GetScanMedia());
                objScanParam.SetImageChannel(m_objProperty.GetImageChannel());
                objScanParam.SetRGBColorDepth(m_objProperty.GetRGBColorDepth());
                objScanParam.SetIRColorDepth(m_objProperty.GetIRColorDepth());
                objScanParam.SetResolution(m_objProperty.GetResolution());
            }
        }

        // Check scanning process
        public ErrorCode ScanCheck()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            m_byExecType = EXEC_SCAN_CHECK;

            // Set MF_BASE01 structure
            errResult = m_objScan.SetBaseSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set scanning parameters for check scanning
            errResult = m_objScan.SetScanSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set MICR parameters
            errResult = m_objMicr.SetMicrSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set MF_PROCESS01 structure
            errResult = m_objOption.SetProcessSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set endorsement type
            errResult = m_objEndorse.SetEndorseStation();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Start check scanning
            errResult = m_objScan.ScanFunction();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            return errResult;
        }

        // Card scanning process
        public ErrorCode ScanCard()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            m_byExecType = EXEC_SCAN_CARD;

            // Clear MF_MICR strucuture
            errResult = m_objMicr.ClearMicrSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set MF_BASE01 strucuture
            errResult = m_objScan.SetBaseSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set card scanning conditions
            errResult = m_objScan.SetScanSetting();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Start card scanning
            errResult = m_objScan.ScanFunction();
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            return errResult;
        }

        // Get MICR data
        public ErrorCode GetMicr(ref CMicrResult objMicrResult)
        {
            return m_objMicr.GetMicr(ref objMicrResult);
        }

        // Get image data
        public ErrorCode GetScanImage(ref CImageResult objImageResult)
        {
            return m_objImage.GetScanImage(ref objImageResult);
        }

        // Error recovery
        public ErrorCode CancelError()
        {
            return m_objProperty.GetMfDevice().CancelError();
        }

        // Get device status
        public void GetDeviceStatus(ref ASB eStatus)
        {
            eStatus = m_objProperty.GetMfDevice().Status;
        }

        // Cancel scanning
        public ErrorCode CancelScan()
        {
            return m_objScan.CancelScan();
        }

        // MICR head cleaning
        public ErrorCode CleaningMicr()
        {
            return m_objMicr.CleaningMicr();
        }

        // Callback function for notifying scanning status
        private void ScanStatus(int nTransactionNumber, MainStatus eMainStatus, ErrorCode eSubStatus, string strPortName)
        {

            if (eSubStatus != ErrorCode.SUCCESS)
            {
                m_cbFuncProcessErrorCB(eSubStatus);
            }

            if (eMainStatus == MainStatus.MF_DATARECEIVE_DONE)
            {
                m_objProperty.SetTransactionNumber(nTransactionNumber);

                if (m_objProperty.GetEndorseType() == PrintingStation.MF_ST_E_ENDORSEMENT)
                {
                    if (m_objProperty.GetPrintText() || m_objProperty.GetPrintImage())
                    {
                        // Notify callback for endorsement
                        m_cbFuncElectronicEndorseCB();
                    }
                }

                if (m_byExecType == EXEC_SCAN_CHECK)
                {
                    // Notify callback for MICR data
                    m_cbFuncMicrCB();
                }
                // Notify callback for image data
                m_cbFuncImageCB();
            }
        }

        ////////////////
        //// Step 4
        ////////////////

        // Register callback for Physical Endorsement
        public void SetPhysicalEndorseCallback(CallbackProcPhysicalEndorse cbPhysicalEndorseCB)
        {
            m_cbFuncPhysicalEndorseCB = cbPhysicalEndorseCB;
        }

        // Register callback for Electronic Endorsement
        public void SetElectronicEndorseCallback(CallbackProcElectronicEndorse cbElectronicEndorseCB)
        {
            m_cbFuncElectronicEndorseCB = cbElectronicEndorseCB;
        }

        // Set endorsement data
        public void SetEndorseParam(ref CEndorseParam objEndorseParam)
        {
            if (objEndorseParam != null)
            {
                m_objProperty.SetEndorseType(objEndorseParam.GetEndorseType());
                m_objProperty.SetPrintText(objEndorseParam.GetPrintText());
                m_objProperty.SetPrintImage(objEndorseParam.GetPrintImage());
            }
        }

        // Get endorsement data
        public void GetEndorseParam(ref CEndorseParam objEndorseParam)
        {
            if (objEndorseParam != null)
            {
                objEndorseParam.SetEndorseType(m_objProperty.GetEndorseType());
                objEndorseParam.SetPrintText(m_objProperty.GetPrintText());
                objEndorseParam.SetPrintImage(m_objProperty.GetPrintImage());
            }
        }

        // Set processing mode options
        public void SetOptionParam(ref COptionParam objOptionParam)
        {
            if (objOptionParam != null)
            {
                m_objProperty.SetEndorsePrintMode(objOptionParam.GetEndorsePrintMode());
                m_objProperty.SetPrnDataUnreceiveCancel(objOptionParam.GetPrnDataUnreceiveCancel());
            }
        }

        // Get processing mode options
        public void GetOptionParam(ref COptionParam objOptionParam)
        {
            if (objOptionParam != null)
            {
                objOptionParam.SetEndorsePrintMode(m_objProperty.GetEndorsePrintMode());
                objOptionParam.SetPrnDataUnreceiveCancel(m_objProperty.GetPrnDataUnreceiveCancel());
            }
        }

        // Reading endorsement data file
        public ErrorCode LoadEndorseFile()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objEndorse.LoadEndorseFile();

            return errResult;
        }

        // Set Physical Endorsement data
        public ErrorCode PhysicalEndorseFunction(uint unTransactionNumber)
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objEndorse.PhysicalEndorseFunction(unTransactionNumber);

            return errResult;
        }

        // Set Electronic Endorsement data
        public ErrorCode ElectronicEndorseFunction()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objEndorse.ElectronicEndorseFunction();

            return errResult;
        }

        // Callback function for notifying endorsement data acceptance
        private void StartEndorsement(int nTransactionNumber, string strPortName)
        {
            if (m_objProperty.GetEndorseType() == PrintingStation.MF_ST_PHYSICAL_ENDORSEMENT)
            {
                if (m_objProperty.GetPrintText() || m_objProperty.GetPrintImage())
                {
                    // notifying endorsement data acceptance
                    m_cbFuncPhysicalEndorseCB((uint)nTransactionNumber);
                }
            }
        }

        // Callback function for notifying endorsement data close
        private void EndEndorsement(int nTransactionNumber, string strPortName)
        {
            /* After this callback is executed, all endorsement data is discarded */
        }
    }
}
